@@ -1,6 +1,7 @@
 /********************************************
 re_cmpl.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2009,2010, Thomas E. Dickey
+copyright 1991-1993,1994, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: re_cmpl.c,v 1.9 2009/09/13 18:43:54 tom Exp $
+ * $MawkId: re_cmpl.c,v 1.22 2010/12/10 17:00:00 tom Exp $
  * @Log: re_cmpl.c,v @
  * Revision 1.6  1994/12/13  00:14:58  mike
  * \\ -> \ on second replacement scan
@@ -47,8 +48,8 @@ the GNU General Public License, version 2, 1991.
 #include "repl.h"
 
 typedef struct re_node {
+    RE_DATA re;			/* keep this first, for re_destroy() */
     STRING *sval;
-    RE_DATA re;
     struct re_node *link;
 } RE_NODE;
 
@@ -95,6 +96,8 @@ re_compile(STRING * sval)
     sval->ref_cnt++;
     p->re.anchored = (*s == '^');
     if (!(p->re.compiled = REcompile(s, sval->len))) {
+	ZFREE(p);
+	sval->ref_cnt--;
 	if (mawk_state == EXECUTION)
 	    rt_error(efmt, REerror(), s);
 	else {			/* compiling */
@@ -112,7 +115,7 @@ re_compile(STRING * sval)
 
 #ifdef DEBUG
     if (dump_RE)
-	REmprint(refRE_DATA(p->re), stderr);
+	REmprint(p->re.compiled, stderr);
 #endif
     return refRE_DATA(p->re);
 }
@@ -134,6 +137,29 @@ re_uncompile(PTR m)
 #endif
 }
 
+void
+re_destroy(PTR m)
+{
+    RE_NODE *p = (RE_NODE *) m;
+    RE_NODE *q;
+    RE_NODE *r;
+
+    if (p != 0) {
+	for (q = re_list, r = 0; q != 0; r = q, q = q->link) {
+	    if (q == p) {
+		free_STRING(p->sval);
+		REdestroy(p->re.compiled);
+		if (r != 0)
+		    r->link = q->link;
+		else
+		    re_list = q->link;
+		free(q);
+		break;
+	    }
+	}
+    }
+}
+
 /*=================================================*/
 /*  replacement	 operations   */
 
@@ -197,13 +223,16 @@ REPL_compile(STRING * sval)
     if (count == 1 && split_buff[0]) {
 	cp->type = C_REPL;
 	cp->ptr = (PTR) split_buff[0];
+	USED_SPLIT_BUFF(0);
     } else {
 	STRING **sp = (STRING **)
 	(cp->ptr = zmalloc(sizeof(STRING *) * count));
 	VCount j = 0;
 
-	while (j < count)
+	while (j < count) {
 	    *sp++ = split_buff[j++];
+	    USED_SPLIT_BUFF(j - 1);
+	}
 
 	cp->type = C_REPLV;
 	cp->vcnt = count;
@@ -220,9 +249,9 @@ repl_destroy(CELL * cp)
     register STRING **p;
     VCount cnt;
 
-    if (cp->type == C_REPL)
+    if (cp->type == C_REPL) {
 	free_STRING(string(cp));
-    else {			/* an C_REPLV           */
+    } else {			/* an C_REPLV           */
 	p = (STRING **) cp->ptr;
 	for (cnt = cp->vcnt; cnt; cnt--) {
 	    if (*p) {
@@ -357,7 +386,7 @@ replv_to_repl(CELL * cp, STRING * sval)
     register STRING **p;
     STRING **sblock = (STRING **) cp->ptr;
     unsigned cnt, vcnt = cp->vcnt;
-    unsigned len;
+    size_t len;
     char *target;
 
 #ifdef DEBUG
@@ -393,3 +422,44 @@ replv_to_repl(CELL * cp, STRING * sval)
     zfree(sblock, vcnt * sizeof(STRING *));
     return cp;
 }
+
+#ifdef NO_LEAKS
+typedef struct _all_ptrs {
+    struct _all_ptrs *next;
+    PTR m;
+} ALL_PTRS;
+
+static ALL_PTRS *all_ptrs;
+/*
+ * Some regular expressions are parsed, and the pointer stored in the byte-code
+ * where we cannot distinguish it from other constants.  Keep a list here, to
+ * free on exit for auditing.
+ */
+void
+no_leaks_re_ptr(PTR m)
+{
+    ALL_PTRS *p = calloc(1, sizeof(ALL_PTRS));
+    p->next = all_ptrs;
+    p->m = m;
+    all_ptrs = p;
+}
+
+void
+re_leaks(void)
+{
+    while (all_ptrs != 0) {
+	ALL_PTRS *next = all_ptrs->next;
+	re_destroy(all_ptrs->m);
+	free(all_ptrs);
+	all_ptrs = next;
+    }
+
+    while (repl_list != 0) {
+	REPL_NODE *p = repl_list->link;
+	free_STRING(repl_list->sval);
+	free_cell_data(repl_list->cp);
+	ZFREE(repl_list);
+	repl_list = p;
+    }
+}
+#endif
