@@ -1,6 +1,8 @@
 /********************************************
 regexp.h
-copyright 1991, Michael D. Brennan
+copyright 2009,2010, Thomas E. Dickey
+copyright 2005, Aleksey Cheusov
+copyright 1991,1993, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +12,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: regexp.h,v 1.8 2009/09/13 17:26:08 tom Exp $
+ * $MawkId: regexp.h,v 1.11 2010/12/10 17:00:00 tom Exp $
  * @Log: regexp.h,v @
  * Revision 1.1.1.1  1993/07/03  18:58:19  mike
  * move source to cvs
@@ -26,8 +28,9 @@ the GNU General Public License, version 2, 1991.
 #include "nstd.h"
 
 PTR REcompile(char *, size_t);
-int REtest(char *, unsigned, PTR);
-char *REmatch(char *, unsigned, PTR, unsigned *);
+void REdestroy(PTR);
+int REtest(char *, size_t, PTR);
+char *REmatch(char *, size_t, PTR, size_t *);
 void REmprint(PTR, FILE *);
 const char *REerror(void);
 
